% This file solves dynamic programming problem for optimal search given inputs specified below
% This function is called by all the outer programs

%psi(s) = theta*[s^(1+sigma)]/(1+sigma)
%psi'(s)=theta*s^sigma
%u(c) = [c^(1-gamma)]/(1-gamma)
%v(c) = [c^(1-gamma)]/(1-gamma)
%delta=0
%r=0

function [D, D_comp, s_0_star, J_0_star, rho_adj_fac, liq_mh_a_an, liq_mh_A_an, unconst_flag] = opt_search_polfn(A0,L,G,G_T,w,a,b,tau,T,B,gridsize,theta,sigma,gamma)

min_U = -Inf;

%specify post-benefit levels of unemp and emp income
b_fixed=0.0*340;
a_fixed=0.25*340;
max_s = 0.25; %maximum search intensity, above which psi(s) = inf.

A_max=max(A0,0)+b+(B<T)*(a+b);
A_min = L+G

%Change grid_inc to alter tradeoff between accuracy/speed
%grid_inc = min((A_max-A_min)/gridsize,5);
grid_inc = (A_max-A_min)/gridsize;

A_t = A_min:grid_inc:A_max;
gridl = length(A_t);
A_t_mat = A_t(ones(1,gridl),:);
A_t_mat = A_t_mat';
A_tp1_mat = A_t_mat';
c_mat = A_t_mat-A_tp1_mat;

A_STAR_VEC=zeros(T,1);
S_STAR_VEC=zeros(T,1);
C_STAR_U=zeros(T,1);
J_STAR=zeros(T,1);
V_STAR=zeros(T,1);
U_STAR=zeros(T,1);
mat_U = zeros(gridl);
opt_A_indices = zeros(T-1,length(A_t));
opt_S = zeros(T,length(A_t));

b_t(1:B)=b;
b_t(B+1:T)=b_fixed;

a_t(1:B)=a+a_fixed;
a_t(B+1:T)=a_fixed;

%Solve for search effort in last period if unemp in that per

V_T=(A_t+a_t(T)+w-tau).^(1-gamma)/(1-gamma);
V_T(A_t+a_t(T)+w-tau<=0)=min_U;
U_T=(A_t+a_t(T)+b_t(T)).^(1-gamma)/(1-gamma);
U_T(A_t+a_t(T)+b_t(T)<=0)=min_U;
opt_S(T,:)=min(((V_T-U_T)/theta).^(1/sigma),1);
J_T=opt_S(T,:).*V_T+(1-opt_S(T,:)).*U_T-theta*[opt_S(T,:).^(1+sigma)]/(1+sigma);

J_t=J_T;

%Replicate J_t in a matrix to facilitate computation
J_t_mat = J_t(ones(1,gridl),:);

V_t_vec(1:gridl)=min_U;
mat_U(:,:)=min_U;


%Backwards induction: go back and solve for period T-1 to period 2:
%V_0=v(A0- AT+w-tau)+ v(AT+w-tau)=(A0-AT+w-tau)^(1-gamma)/(1-gamma)+V_1
%U_0=u(A0- AT+b_t(t))+ J_1=(A0+b_t(t))^(1-gamma)/(1-gamma)+J_1
for t = T-1:-1:2
         c_emp_t = (A_t+sum(a_t(t:T)))/(T-t+1)+w-tau;
         c_emp_pos_t = c_emp_t>0;
         A_t_feas=(A_t>=L|t<=G_T);
         V_t_vec(c_emp_pos_t&A_t_feas)=(T-t+1)*(c_emp_t(c_emp_pos_t&A_t_feas).^(1-gamma))/(1-gamma); 
         %period t: rows of matrix U, period t+1: columns of matrix U
         %set value to min_U if negative consumption
         c_mat_t=c_mat+a_t(t)+b_t(t);
         c_pos_t=c_mat_t>0;
         A_t_feas_mat=(A_t_mat>=L|t<=G_T);
         mat_U(c_pos_t&A_t_feas_mat)=((c_mat_t(c_pos_t&A_t_feas_mat)).^(1-gamma))/(1-gamma)+J_t_mat(c_pos_t&A_t_feas_mat);

         %find the max val in each row of U, generating a vector
         %Note: max function takes column-max, so need to transpose
         %find the optimal A_t+1 which corresponds for each A_t
      
         [U_t_vec, opt_A_indices(t,:)]=max(mat_U');
         opt_S(t,:)=min(((V_t_vec-U_t_vec)/theta).^(1/sigma),max_s);
         J_t=opt_S(t,:).*V_t_vec+(1-opt_S(t,:)).*U_t_vec-theta*[opt_S(t,:).^(1+sigma)]/(1+sigma);
         J_t_mat = J_t(ones(1,gridl),:);
     
end

c_0_e = (A0+sum(a_t))/T+w-tau;
V_0=T*(c_0_e^(1-gamma))/(1-gamma);
vec_U0=((A0-A_t+a_t(1)+b_t(1)).^(1-gamma))/(1-gamma)+J_t;
vec_U0(A0-A_t+a_t(1)+b_t(1)<=0)=min_U;
[U_0, maxindex_tm1] = max(vec_U0);


%Recover optimal policy functions

A_STAR_VEC(1)=A0;
S_STAR_VEC(1)=min(((V_0-U_0)/theta)^(1/sigma),max_s);
C_STAR_E(1) = c_0_e;

%(V_0-U_0)/theta
%pause

A_STAR_VEC(2)= A_t(maxindex_tm1);
S_STAR_VEC(2)= opt_S(2,maxindex_tm1);
C_STAR_E(2) = (A_STAR_VEC(2)+sum(a_t(2:T)))/(T-2+1)+w-tau;

for t=3:T
    maxindex_tm1=opt_A_indices(t-1,maxindex_tm1);
    A_STAR_VEC(t)=A_t(maxindex_tm1);
    S_STAR_VEC(t)= opt_S(t,maxindex_tm1);
    C_STAR_E(t) =  (A_STAR_VEC(t)+sum(a_t(t:T)))/(T-t+1)+w-tau;
end
C_STAR_U = A_STAR_VEC - [A_STAR_VEC(2:T); 0] + a_t' + b_t';

%A_STAR_VEC(1:min(T,100))
%C_STAR_E(1:min(T,50))'
%C_STAR_U(1:min(T,100))
%S_STAR_VEC(1:min(T,100))

if max(A_STAR_VEC(2:T))==A_max
    disp('WARNING: ASSET CHOICE HIT UPPER CORNER')
    pause
end

unconst_flag=0;
if min(A_STAR_VEC(2:T))>A_min
    disp('Note: BORROWING CONSTRAINT NEVER BINDS --> dG_dw = 0')
    unconst_flag=1
end

s_0_star = S_STAR_VEC(1);

if max(S_STAR_VEC)>=max_s
    period = min(find(~(S_STAR_VEC<max_s)));
    disp(['Note: SEARCH EFFORT HIT max_s in PERIOD ' num2str(period)])        
end


J_0_star=S_STAR_VEC(1)*V_0+(1-S_STAR_VEC(1))*U_0-theta*[S_STAR_VEC(1)^(1+sigma)]/(1+sigma);


%ANALYTICAL CHECKS (NOT ESSENTIAL FOR SOLVING THE DP PROBLEM)

surv_prob = 1-S_STAR_VEC;
D=0;
for t = 1:T
    D = D+prod(surv_prob(1:t));
end

D_comp=0;
Econs_unemp=0;
for t = 1:B
    D_comp = D_comp+prod(surv_prob(1:t));
    Econs_unemp=Econs_unemp+prod(surv_prob(1:t))*C_STAR_U(t);
end
avgcons_unemp = Econs_unemp/D;

%calculate rho
margu0 = C_STAR_E(1)^-gamma;
Emargu_t = s_0_star*T*(C_STAR_E(1)^-gamma);
Econs_emp =  s_0_star*T*(C_STAR_E(1));
Eemp_dur = s_0_star*T
for t = 2:T
    Emargu_t = Emargu_t+prod(surv_prob(1:t-1))*S_STAR_VEC(t)*(T-t+1)*(C_STAR_E(t)^-gamma);
    Econs_emp = Econs_emp+ prod(surv_prob(1:t-1))*S_STAR_VEC(t)*(T-t+1)*(C_STAR_E(t));
    Eemp_dur = Eemp_dur +prod(surv_prob(1:t-1))*S_STAR_VEC(t)*(T-t+1);
end
Emargu_T = Emargu_t/Eemp_dur;
rho = (Emargu_T-margu0)/Emargu_T;

avgcons_emp = Econs_emp/Eemp_dur;
avg_cons_drop_T = (avgcons_emp-avgcons_unemp)/avgcons_emp;

%calculate rho_B
margu0 = C_STAR_E(1)^-gamma;
Emargu_t = s_0_star*T*(C_STAR_E(1)^-gamma);
Econs_emp =  s_0_star*T*(C_STAR_E(1));
Eemp_dur_B = s_0_star*T;
for t = 2:B
    Emargu_t = Emargu_t+prod(surv_prob(1:t-1))*S_STAR_VEC(t)*(T-t+1)*(C_STAR_E(t)^-gamma);
    Econs_emp = Econs_emp+ prod(surv_prob(1:t-1))*S_STAR_VEC(t)*(T-t+1)*(C_STAR_E(t));
    Eemp_dur_B = Eemp_dur_B +prod(surv_prob(1:t-1))*S_STAR_VEC(t)*(T-t+1);
end
Emargu_B = Emargu_t/Eemp_dur_B;
rho_B = (Emargu_B-margu0)/Emargu_B;
rho_w = (Emargu_B/Emargu_T)*((D_comp-rho_B*B)/(D-rho*T))*(D/D_comp)-1;
rho_adj_fac = ((D-rho*T)/(T-D))*((1+rho_w)*D_comp/D);

avgcons_emp = Econs_emp/Eemp_dur_B;
avg_cons_drop_B = (avgcons_emp-avgcons_unemp)/avgcons_emp;

%Calculate ds_da/ds_dw analytically
dV0_dw = B*C_STAR_E(1)^-gamma;
dU0_dw = s_0_star*B*(C_STAR_E(1)^-gamma);
for t = 2:B
    dU0_dw = dU0_dw+prod(surv_prob(1:t-1))*S_STAR_VEC(t)*(B-t+1)*(C_STAR_E(t)^-gamma);
end
dU0_db = (C_STAR_U(1)^-gamma);
for t = 2:B
    dU0_db=dU0_db+prod(surv_prob(2:t))*(C_STAR_U(t)^-gamma);
end
liq = dV0_dw-dU0_dw-dU0_db;
mh =  dV0_dw-dU0_dw;
ben = -dU0_db;
liq_mh_a_an = -liq/mh

dV0_dA = C_STAR_E(1)^-gamma;
dU0_dA = C_STAR_U(1)^-gamma;
liq_A = dV0_dA-dU0_dA;
liq_mh_A_an_old = -(liq_A/mh)*(B*(B<T)+(B==T)*D/(1-s_0_star))
liq_mh_A_an = -(liq_A*B/(liq_A*B-ben))

surv_rates = cumprod(surv_prob);

%Use command below to check consumption path if desired
%params_for_cpath_below = [G b]
%[(1:100)' C_STAR_E(1:100)' C_STAR_U(1:100) A_STAR_VEC(1:100)/1000]